<?php

namespace Litecms\Groupchat\Http\Controllers;

use App\Http\Controllers\ResourceController as BaseController;
use Form;
use Litecms\Groupchat\Http\Requests\GroupchatRequest;
use Litecms\Groupchat\Interfaces\GroupchatRepositoryInterface;
use Litecms\Groupchat\Models\Groupchat;

/**
 * Resource controller class for groupchat.
 */
class GroupchatResourceController extends BaseController
{

    /**
     * Initialize groupchat resource controller.
     *
     * @param type GroupchatRepositoryInterface $groupchat
     *
     * @return null
     */
    public function __construct(GroupchatRepositoryInterface $groupchat)
    {
        parent::__construct();
        $this->repository = $groupchat;
        $this->repository
            ->pushCriteria(\Litepie\Repository\Criteria\RequestCriteria::class)
            ->pushCriteria(\Litecms\Groupchat\Repositories\Criteria\GroupchatResourceCriteria::class);
    }

    /**
     * Display a list of groupchat.
     *
     * @return Response
     */
    public function index(GroupchatRequest $request)
    {
        $view = $this->response->theme->listView();

        if ($this->response->typeIs('json')) {
            $function = camel_case('get-' . $view);
            return $this->repository
                ->setPresenter(\Litecms\Groupchat\Repositories\Presenter\GroupchatPresenter::class)
                ->$function();
        }

        $groupchats = $this->repository->paginate();

        return $this->response->title(trans('groupchat::groupchat.names'))
            ->view('groupchat::groupchat.index', true)
            ->data(compact('groupchats'))
            ->output();
    }

    /**
     * Display groupchat.
     *
     * @param Request $request
     * @param Model   $groupchat
     *
     * @return Response
     */
    public function show(GroupchatRequest $request, Groupchat $groupchat)
    {

        if ($groupchat->exists) {
            $view = 'groupchat::groupchat.show';
        } else {
            $view = 'groupchat::groupchat.new';
        }

        return $this->response->title(trans('app.view') . ' ' . trans('groupchat::groupchat.name'))
            ->data(compact('groupchat'))
            ->view($view, true)
            ->output();
    }

    /**
     * Show the form for creating a new groupchat.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(GroupchatRequest $request)
    {

        $groupchat = $this->repository->newInstance([]);
        return $this->response->title(trans('app.new') . ' ' . trans('groupchat::groupchat.name')) 
            ->view('groupchat::groupchat.create', true) 
            ->data(compact('groupchat'))
            ->output();
    }

    /**
     * Create new groupchat.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(GroupchatRequest $request)
    {
        try {
            $attributes              = $request->all();
            $attributes['user_id']   = user_id();
            $attributes['user_type'] = user_type();
            $groupchat                 = $this->repository->create($attributes);

            return $this->response->message(trans('messages.success.created', ['Module' => trans('groupchat::groupchat.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('groupchat/groupchat/' . $groupchat->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/groupchat/groupchat'))
                ->redirect();
        }

    }

    /**
     * Show groupchat for editing.
     *
     * @param Request $request
     * @param Model   $groupchat
     *
     * @return Response
     */
    public function edit(GroupchatRequest $request, Groupchat $groupchat)
    {
        return $this->response->title(trans('app.edit') . ' ' . trans('groupchat::groupchat.name'))
            ->view('groupchat::groupchat.edit', true)
            ->data(compact('groupchat'))
            ->output();
    }

    /**
     * Update the groupchat.
     *
     * @param Request $request
     * @param Model   $groupchat
     *
     * @return Response
     */
    public function update(GroupchatRequest $request, Groupchat $groupchat)
    {
        try {
            $attributes = $request->all();

            $groupchat->update($attributes);
            return $this->response->message(trans('messages.success.updated', ['Module' => trans('groupchat::groupchat.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('groupchat/groupchat/' . $groupchat->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('groupchat/groupchat/' . $groupchat->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the groupchat.
     *
     * @param Model   $groupchat
     *
     * @return Response
     */
    public function destroy(GroupchatRequest $request, Groupchat $groupchat)
    {
        try {

            $groupchat->delete();
            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('groupchat::groupchat.name')]))
                ->code(202)
                ->status('success')
                ->url(guard_url('groupchat/groupchat/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('groupchat/groupchat/' . $groupchat->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove multiple groupchat.
     *
     * @param Model   $groupchat
     *
     * @return Response
     */
    public function delete(GroupchatRequest $request, $type)
    {
        try {
            $ids = hashids_decode($request->input('ids'));

            if ($type == 'purge') {
                $this->repository->purge($ids);
            } else {
                $this->repository->delete($ids);
            }

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('groupchat::groupchat.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('groupchat/groupchat'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/groupchat/groupchat'))
                ->redirect();
        }

    }

    /**
     * Restore deleted groupchats.
     *
     * @param Model   $groupchat
     *
     * @return Response
     */
    public function restore(GroupchatRequest $request)
    {
        try {
            $ids = hashids_decode($request->input('ids'));
            $this->repository->restore($ids);

            return $this->response->message(trans('messages.success.restore', ['Module' => trans('groupchat::groupchat.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('/groupchat/groupchat'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/groupchat/groupchat/'))
                ->redirect();
        }

    }

}
